'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (babel) {
  var t = babel.types,
      version = babel.version;

  var babel6 = _semver2.default.satisfies(version, '^6.0.0');

  return {
    name: 'babel-blade', // not required
    inherits: babel6 ? _babelPluginSyntaxJsx2.default : _pluginSyntaxJsx2.default, // for jsx/babel6-7 interop
    visitor: {
      Identifier(path) {
        handleCreateRazor(path, t);
      }
    }
  };
};

exports.handleCreateRazor = handleCreateRazor;

var _semver = require('semver');

var _semver2 = _interopRequireDefault(_semver);

var _pluginSyntaxJsx = require('@babel/plugin-syntax-jsx');

var _pluginSyntaxJsx2 = _interopRequireDefault(_pluginSyntaxJsx);

var _babelPluginSyntaxJsx = require('babel-plugin-syntax-jsx');

var _babelPluginSyntaxJsx2 = _interopRequireDefault(_babelPluginSyntaxJsx);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// data structure
var _require = require('./dataStructures'),
    RazorData = _require.RazorData; // for jsx/babel6-7 interop


var _require2 = require('./helpers'),
    isObject = _require2.isObject,
    isCallee = _require2.isCallee,
    isCreateQuery = _require2.isCreateQuery,
    isCreateFragment = _require2.isCreateFragment,
    getAssignTarget = _require2.getAssignTarget,
    getCalleeArgs = _require2.getCalleeArgs,
    maybeGetSimpleString = _require2.maybeGetSimpleString,
    getSimpleFragmentName = _require2.getSimpleFragmentName;

var _require3 = require('./semanticTrace'),
    semanticTrace = _require3.semanticTrace;

/****
 *
 * Discussion of babel strategy
 *
 * at the top level we give users createQuery and createFragment methods.
 * when these are called and assigned, whatever identifier they get assigned to becomes a "razor".
 *
 * the strategy is
 *
 * - declare an `aliasReplaceQueue` which is a `Map()`
 * - semanticTraverse entire AST
 *   - read into datastructure
 *   - where renaming will be needed, push the node into `aliasReplaceQueue`
 * - inject graphql
 * - `aliasReplaceQueue.forEach` and rename
 *
 * LHS cases to handle:
 * - const var1 = DATA
 * - const { var2 } = DATA
 * - const { var3, var4 } = DATA
 * - const { var5: var6  } = DATA
 * - const { var7: { var8: var9} } = DATA
 *
 * interaction cases to handle:
 * - const var1 = DATA.v2
 * - const { var2 } = DATA.v2
 * - const { var3, var4 } = DATA.v2
 * - const { var5: var6  } = DATA.v2
 * - const { var7: { var8: var9} } = DATA.v2
 *
 * RHS cases
 * - const var1 = DATA.v3
 * - const var2 = DATA.v3.var3
 * - const var4 = DATA.v3.var5
 * - const var6 = DATA.v3({ foo: 1, bar: 2})
 * - const var7 = DATA.v3({ foo: 3, bar: 4}).var8
 * - DATA.v3.var8 // no assignment!
 *
 * Array methods
 * - const var2 = DATA.v3.var3[0].foo
 * - DATA.foo.map(bar => bar.baz)
 * - DATA.foo.map(({bar}) => bar.baz)
 * - DATA.foo.map(function ({bar}) {bar.baz})
 *
 **/

/* eslint-disable complexity */
/* eslint-disable babel/new-cap */

function handleCreateRazor(path, t) {
  if (isCreateQuery(path) || isCreateFragment(path)) {
    // get the identifier and available args
    var identifier = getAssignTarget(path);
    var queryArgs = void 0;
    if (isCallee(path)) queryArgs = getCalleeArgs(path);
    // traverse scope for identifier references
    var refs = path.scope.bindings[identifier].referencePaths;
    // clear the reference
    var razorParentPath = path.findParent(function (ppath) {
      return ppath.isVariableDeclaration();
    });
    if (!razorParentPath.parentPath.isExportNamedDeclaration()) {
      razorParentPath.remove(); // remove it unless its exported :)
    }
    // create the queue - we will defer alias replacement til all semantic traversals are done
    var aliasReplaceQueue = new Map();
    if (refs.length > 0) {
      var razorID = null;
      if (isCreateFragment(path) && !queryArgs[0]) throw new Error('createFragment must have one argument to specify the graphql type they are on');
      var fragmentType = isCreateFragment(path) && maybeGetSimpleString(queryArgs[0]); //getFragmentName(path)
      var queryType = isCreateFragment(path) ? 'fragment' : 'query';
      var razorData = new RazorData({
        type: queryType,
        name: isCreateFragment(path) ? t.Identifier(identifier) : identifier,
        fragmentType,
        args: isCreateQuery(path) && queryArgs
      });

      // eslint-disable-next-line
      function idempotentAddToRazorData(semPath) {
        var currentRazor = razorData;
        semPath.forEach(function (_ref) {
          var name = _ref[0],
              ref = _ref[1];

          var aliasPath = void 0,
              calleeArguments = void 0;
          if (isCallee(ref)) {
            // if its a callee, extract its args and push it into RHS
            // will parse out fragments/args/directives later
            calleeArguments = getCalleeArgs(ref);
            aliasPath = ref;
          }
          var args = [];
          var fragments = [];
          var directives = [];

          if (calleeArguments) {
            for (var _iterator = calleeArguments, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
              var _ref2;

              if (_isArray) {
                if (_i >= _iterator.length) break;
                _ref2 = _iterator[_i++];
              } else {
                _i = _iterator.next();
                if (_i.done) break;
                _ref2 = _i.value;
              }

              var x = _ref2;

              if (x.type === 'StringLiteral' || x.type === 'TemplateLiteral') {
                // its an arg or a directive; peek at first character to decide
                var peek = x.quasis ? x.quasis[0].value.raw[0] : x.value[0];
                peek === '@' ? directives.push(x) : args.push(x);
              } else {
                // its a fragment
                fragments.push(x);
              }
            }
          }
          // const mockRazorToGetAlias = new BladeData({name, args}) // this is hacky, i know; a result of the datastructures being legacy
          /*console.log('b4',{name, 
                           args: args.length && args[0].value, 
                           currentRazor: [...currentRazor._children],
                           razorData: [...razorData._children],
                          })*/
          currentRazor = currentRazor.add({
            name,
            args,
            directives,
            fragments
          });
          /*console.log('aftr',{
                           currentRazor: [...currentRazor._children],
                           razorData: [...razorData._children],
                          })*/
          if (currentRazor._args && aliasPath) {
            aliasReplaceQueue.set(aliasPath, currentRazor);
          }
        });
      }

      refs.forEach(function (razor) {
        // define visitor
        var semanticVisitor = {
          CallExpression() {
            for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
              args[_key] = arguments[_key];
            }

            // const [hand, ref, semPath, ...rest] = args
            var ref = args[1];

            var callee = ref.get('callee');
            // console.log('CallExpression', hand, semPath, ref,callee)
            ref.replaceWith(callee);
          },
          Identifier() {
            for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
              args[_key2] = arguments[_key2];
            }

            // const [hand, ref, semPath, ...rest] = args
            var hand = args[0],
                semPath = args[2];
            // console.log("Identifier", hand, semPath, ref);

            if (hand === 'origin') idempotentAddToRazorData(semPath);
          },
          MemberExpression() {
            for (var _len3 = arguments.length, topargs = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
              topargs[_key3] = arguments[_key3];
            }

            // const [hand, ref, semPath, ...rest] = topargs
            var semPath = topargs[2];
            // console.log('MemberExpression', hand, semPath, ref)

            idempotentAddToRazorData(semPath);
          }
        };
        // go through all razors
        if (isCallee(razor)) {
          // we have been activated! time to make a blade!
          razorID = getAssignTarget(razor);
          // clear the reference
          if (razor.container.arguments[0]) razor.parentPath.replaceWith(razor.container.arguments[0]);else razor.parentPath.remove();
          // extract data
          semanticTrace(razor, razorID, semanticVisitor);
        }
      });
      // insert query
      refs.forEach(function (razor) {
        if (!isObject(razor)) {
          var _razorData$print = razorData.print(),
              stringAccumulator = _razorData$print.stringAccumulator,
              litAccumulator = _razorData$print.litAccumulator;

          var graphqlOutput = t.templateLiteral(stringAccumulator.map(function (str) {
            return t.templateElement({ raw: str, cooked: str });
          }), litAccumulator.map(function (lit) {
            if (lit.isFragment) {
              // we tagged this inside BladeData
              return t.callExpression(lit, [t.stringLiteral(getSimpleFragmentName(lit))]);
            }
            return lit || t.nullLiteral();
          }));

          if (razor.isExportNamedDeclaration()) {
            // allow 1 export
            var decs = razor.get('declaration').get('declarations');
            if (decs.length > 1) throw new Error('detected multiple export declarations in one line, you are restricted to 1 for now');
            razor = decs[0].get('init'); // mutate razor to get ready for replacement
          }
          if (razorData._type === 'fragment') {
            razor.replaceWith(t.arrowFunctionExpression([t.identifier(identifier)], graphqlOutput));
          } else {
            razor.replaceWith(graphqlOutput);
          }
        }
      });
    }
    aliasReplaceQueue.forEach(function (currentRazor, aliasPath) {
      if (currentRazor._alias) {
        aliasPath.parentPath.replaceWith(aliasPath);
        aliasPath.node.property.name = currentRazor._alias;
      }
    });
  }
}